import org.jblas.DoubleMatrix;
import org.jblas.FloatMatrix;

import org.shogun.modshogun;
import org.shogun.RealVector;
import org.shogun.BinaryLabels;
import org.shogun.AveragedPerceptron;
import org.shogun.AccuracyMeasure;
import org.shogun.Labels;
import org.shogun.WrappedObjectArray;
import org.shogun.SerializableAsciiFile;
import org.shogun.CSVFile;
import org.shogun.RealFeatures;

public class averaged_perceptron {
static {
System.loadLibrary("modshogun");
}

public static void main(String argv[]) {
modshogun.init_shogun_with_defaults();

CSVFile f_feats_train = new CSVFile("../../data/classifier_binary_2d_linear_features_train.dat");
CSVFile f_feats_test = new CSVFile("../../data/classifier_binary_2d_linear_features_test.dat");
CSVFile f_labels_train = new CSVFile("../../data/classifier_binary_2d_linear_labels_train.dat");
CSVFile f_labels_test = new CSVFile("../../data/classifier_binary_2d_linear_labels_test.dat");

//![create_features]
RealFeatures features_train = new RealFeatures(f_feats_train);
RealFeatures features_test = new RealFeatures(f_feats_test);
BinaryLabels labels_train = new BinaryLabels(f_labels_train);
BinaryLabels labels_test = new BinaryLabels(f_labels_test);
//![create_features]

//![set_parameters]
double learn_rate = 1.0;
int max_iter = 1000;
AveragedPerceptron perceptron = new AveragedPerceptron(features_train, labels_train);
perceptron.set_learn_rate(learn_rate);
perceptron.set_max_iter(max_iter);
//![set_parameters]

//![train_and_apply]
perceptron.train();
perceptron.set_features(features_test);
Labels labels_predict = perceptron.apply();
//![train_and_apply]

//![extract_weights]
DoubleMatrix weights = perceptron.get_w();
double bias = perceptron.get_bias();
//![extract_weights]

//![evaluate_accuracy]
AccuracyMeasure eval = new AccuracyMeasure();
double accuracy = eval.evaluate(labels_predict, labels_test);
//![evaluate_accuracy]

// Serialize output for integration testing (automatically generated)
WrappedObjectArray __sg_storage = new WrappedObjectArray();
SerializableAsciiFile __sg_storage_file = new SerializableAsciiFile("averaged_perceptron.dat", 'w');
__sg_storage.append_wrapped_real(learn_rate, "learn_rate");
__sg_storage.append_wrapped_int(max_iter, "max_iter");
__sg_storage.append_wrapped_real_vector(weights, "weights");
__sg_storage.append_wrapped_real(bias, "bias");
__sg_storage.append_wrapped_real(accuracy, "accuracy");
__sg_storage.save_serializable(__sg_storage_file);

}
}
