import org.jblas.DoubleMatrix;
import org.jblas.FloatMatrix;

import org.shogun.modshogun;
import org.shogun.RealVector;
import org.shogun.BinaryLabels;
import org.shogun.AccuracyMeasure;
import org.shogun.WrappedObjectArray;
import org.shogun.SerializableAsciiFile;
import org.shogun.RealFeatures;
import org.shogun.CSVFile;
import org.shogun.GaussianKernel;
import org.shogun.LibSVM;

public class kernel_svm {
static {
System.loadLibrary("modshogun");
}

public static void main(String argv[]) {
modshogun.init_shogun_with_defaults();

CSVFile f_feats_train = new CSVFile("../../data/classifier_binary_2d_nonlinear_features_train.dat");
CSVFile f_feats_test = new CSVFile("../../data/classifier_binary_2d_nonlinear_features_test.dat");
CSVFile f_labels_train = new CSVFile("../../data/classifier_binary_2d_nonlinear_labels_train.dat");
CSVFile f_labels_test = new CSVFile("../../data/classifier_binary_2d_nonlinear_labels_test.dat");

//![create_features]
RealFeatures features_train = new RealFeatures(f_feats_train);
RealFeatures features_test = new RealFeatures(f_feats_test);
BinaryLabels labels_train = new BinaryLabels(f_labels_train);
BinaryLabels labels_test = new BinaryLabels(f_labels_test);
//![create_features]

//![set_parameters]
double C = 1.0;
double epsilon = 0.001;
GaussianKernel gauss_kernel = new GaussianKernel(features_train, features_train, 15);
//![set_parameters]

//![create_instance]
LibSVM svm = new LibSVM(C, gauss_kernel, labels_train);
svm.set_epsilon(epsilon);
//![create_instance]

//![train_and_apply]
svm.train();
BinaryLabels labels_predict = svm.apply_binary(features_test);
//![train_and_apply]

//![extract_weights_bias]
DoubleMatrix alphas = svm.get_alphas();
double b = svm.get_bias();
//![extract_weights_bias]

//![evaluate_accuracy]
AccuracyMeasure eval = new AccuracyMeasure();
double accuracy = eval.evaluate(labels_predict, labels_test);
//![evaluate_accuracy]

// additional integration testing variables
DoubleMatrix output = labels_predict.get_labels();

// Serialize output for integration testing (automatically generated)
WrappedObjectArray __sg_storage = new WrappedObjectArray();
SerializableAsciiFile __sg_storage_file = new SerializableAsciiFile("kernel_svm.dat", 'w');
__sg_storage.append_wrapped_real(C, "C");
__sg_storage.append_wrapped_real(epsilon, "epsilon");
__sg_storage.append_wrapped_real_vector(alphas, "alphas");
__sg_storage.append_wrapped_real(b, "b");
__sg_storage.append_wrapped_real(accuracy, "accuracy");
__sg_storage.append_wrapped_real_vector(output, "output");
__sg_storage.save_serializable(__sg_storage_file);

}
}
