import org.jblas.DoubleMatrix;
import org.jblas.FloatMatrix;

import org.shogun.modshogun;
import org.shogun.GaussianProcessClassification;
import org.shogun.MulticlassLabels;
import org.shogun.MultiLaplaceInferenceMethod;
import org.shogun.RealVector;
import org.shogun.MulticlassAccuracy;
import org.shogun.SoftMaxLikelihood;
import org.shogun.WrappedObjectArray;
import org.shogun.ConstMean;
import org.shogun.SerializableAsciiFile;
import org.shogun.CSVFile;
import org.shogun.Math;
import org.shogun.GaussianKernel;
import org.shogun.RealFeatures;

public class gaussian_process_classifier {
static {
System.loadLibrary("modshogun");
}

public static void main(String argv[]) {
modshogun.init_shogun_with_defaults();

CSVFile f_feats_train = new CSVFile("../../data/classifier_4class_2d_linear_features_train.dat");
CSVFile f_feats_test = new CSVFile("../../data/classifier_4class_2d_linear_features_test.dat");
CSVFile f_labels_train = new CSVFile("../../data/classifier_4class_2d_linear_labels_train.dat");
CSVFile f_labels_test = new CSVFile("../../data/classifier_4class_2d_linear_labels_test.dat");
org.shogun.Math.init_random(1);

//![create_features]
RealFeatures features_train = new RealFeatures(f_feats_train);
RealFeatures features_test = new RealFeatures(f_feats_test);
MulticlassLabels labels_train = new MulticlassLabels(f_labels_train);
MulticlassLabels labels_test = new MulticlassLabels(f_labels_test);
//![create_features]

//![create_appropriate_kernel_and_mean_function]
GaussianKernel kernel = new GaussianKernel(2.0);
ConstMean mean_function = new ConstMean();
//![create_appropriate_kernel_and_mean_function]

//![create_instance]
SoftMaxLikelihood gauss_likelihood = new SoftMaxLikelihood();
MultiLaplaceInferenceMethod inference_method = new MultiLaplaceInferenceMethod(kernel, features_train, mean_function, labels_train, gauss_likelihood);
GaussianProcessClassification gp_classifier = new GaussianProcessClassification(inference_method);
//![create_instance]

//![train_and_apply]
gp_classifier.train();
MulticlassLabels labels_predict = gp_classifier.apply_multiclass(features_test);
//![train_and_apply]

//![extract_the_probabilities]
DoubleMatrix prob = gp_classifier.get_probabilities(features_train);
//![extract_the_probabilities]

//![evaluate_accuracy]
MulticlassAccuracy evals = new MulticlassAccuracy();
double accuracy = evals.evaluate(labels_predict, labels_test);
//![evaluate_accuracy]

// integration testing variables
DoubleMatrix output = labels_test.get_labels();

// Serialize output for integration testing (automatically generated)
WrappedObjectArray __sg_storage = new WrappedObjectArray();
SerializableAsciiFile __sg_storage_file = new SerializableAsciiFile("gaussian_process_classifier.dat", 'w');
__sg_storage.append_wrapped_real_vector(prob, "prob");
__sg_storage.append_wrapped_real(accuracy, "accuracy");
__sg_storage.append_wrapped_real_vector(output, "output");
__sg_storage.save_serializable(__sg_storage_file);

}
}
