import org.jblas.DoubleMatrix;
import org.jblas.FloatMatrix;

import org.shogun.modshogun;
import org.shogun.FITCInferenceMethod;
import org.shogun.MeanSquaredError;
import org.shogun.RealVector;
import org.shogun.GaussianProcessRegression;
import org.shogun.WrappedObjectArray;
import org.shogun.GaussianLikelihood;
import org.shogun.SerializableAsciiFile;
import org.shogun.ZeroMean;
import org.shogun.RegressionLabels;
import org.shogun.CSVFile;
import org.shogun.IntVector;
import org.shogun.GaussianKernel;
import org.shogun.RealFeatures;

public class sparse_gaussian_process_regression {
static {
System.loadLibrary("modshogun");
}

public static void main(String argv[]) {
modshogun.init_shogun_with_defaults();

CSVFile f_feats_train = new CSVFile("../../data/regression_1d_sinc_features_train.dat");
CSVFile f_feats_inducing = new CSVFile("../../data/regression_1d_sinc_features_train.dat");
CSVFile f_feats_test = new CSVFile("../../data/regression_1d_sinc_features_test.dat");
CSVFile f_labels_train = new CSVFile("../../data/regression_1d_sinc_labels_train.dat");
CSVFile f_labels_test = new CSVFile("../../data/regression_1d_sinc_labels_test.dat");

//![create_features]
RealFeatures features_train = new RealFeatures(f_feats_train);
RealFeatures features_test = new RealFeatures(f_feats_test);
RegressionLabels labels_train = new RegressionLabels(f_labels_train);
RegressionLabels labels_test = new RegressionLabels(f_labels_test);
//![create_features]

//![create_kernel_and_mean_function]
double width = 1.0;
GaussianKernel kernel = new GaussianKernel(features_train, features_train, width);
ZeroMean mean_function = new ZeroMean();
//![create_kernel_and_mean_function]

//![create_inference]
DoubleMatrix inducing_points = new DoubleMatrix(1, 3);
inducing_points.put(0, 0);
inducing_points.put(1, 1);
inducing_points.put(2, 2);
RealFeatures inducing_features = new RealFeatures(f_feats_inducing);
inducing_features.add_subset(inducing_points);

GaussianLikelihood gauss_likelihood = new GaussianLikelihood();
FITCInferenceMethod inference_method = new FITCInferenceMethod(kernel, features_train, mean_function, labels_train, gauss_likelihood, inducing_features);
//![create_inference]

//![create_instance]
GaussianProcessRegression gp_regression = new GaussianProcessRegression(inference_method);
//![create_instance]

//![train_and_apply]
gp_regression.train();
RegressionLabels labels_predict = gp_regression.apply_regression(features_test);
//![train_and_apply]

//![compute_variance]
DoubleMatrix variances = gp_regression.get_variance_vector(features_test);
//![compute_variance]

//![evaluate_error]
MeanSquaredError mserror = new MeanSquaredError();
double mse = mserror.evaluate(labels_predict, labels_test);
//![evaluate_error]

// integration testing variables
DoubleMatrix output = labels_test.get_labels();

// Serialize output for integration testing (automatically generated)
WrappedObjectArray __sg_storage = new WrappedObjectArray();
SerializableAsciiFile __sg_storage_file = new SerializableAsciiFile("sparse_gaussian_process_regression.dat", 'w');
__sg_storage.append_wrapped_real(width, "width");
__sg_storage.append_wrapped_real_vector(variances, "variances");
__sg_storage.append_wrapped_real(mse, "mse");
__sg_storage.append_wrapped_real_vector(output, "output");
__sg_storage.save_serializable(__sg_storage_file);

}
}
