import org.jblas.DoubleMatrix;
import org.jblas.FloatMatrix;

import org.shogun.modshogun;
import org.shogun.MeanSquaredError;
import org.shogun.Math;
import org.shogun.NeuralLayers;
import org.shogun.NeuralNetwork;
import org.shogun.DynamicObjectArray;
import org.shogun.CSVFile;
import org.shogun.RegressionLabels;
import org.shogun.WrappedObjectArray;
import org.shogun.SerializableAsciiFile;
import org.shogun.RealVector;
import org.shogun.RealFeatures;

public class feedforward_net_regression {
static {
System.loadLibrary("modshogun");
}

public static void main(String argv[]) {
modshogun.init_shogun_with_defaults();

CSVFile f_feats_train = new CSVFile("../../data/regression_1d_sinc_features_train_with_9d_noise.dat");
CSVFile f_feats_test = new CSVFile("../../data/regression_1d_sinc_features_test_with_9d_noise.dat");
CSVFile f_labels_train = new CSVFile("../../data/regression_1d_sinc_labels_train.dat");
CSVFile f_labels_test = new CSVFile("../../data/regression_1d_sinc_labels_test.dat");

org.shogun.Math.init_random(1);

//![create_features]
RealFeatures features_train = new RealFeatures(f_feats_train);
RealFeatures features_test = new RealFeatures(f_feats_test);
RegressionLabels labels_train = new RegressionLabels(f_labels_train);
RegressionLabels labels_test = new RegressionLabels(f_labels_test);
//![create_features]

//![add_layers]
int dimensions = features_train.get_num_features();
NeuralLayers layers = new NeuralLayers();
layers = layers.input(dimensions);
layers = layers.rectified_linear(20);
layers = layers.linear(1);
DynamicObjectArray all_layers = layers.done();
//![add_layers]

//![create_instance]
NeuralNetwork network = new NeuralNetwork(all_layers);
network.quick_connect();
network.initialize_neural_network();
//![create_instance]

//![set_parameters]
network.set_l2_coefficient(0.1);
network.set_max_num_epochs(40);
network.set_epsilon(0.0);
network.set_gd_learning_rate(0.1);
network.set_gd_momentum(0.9);
//![set_parameters]

//![train_and_apply]
network.set_labels(labels_train);
network.train(features_train);
RegressionLabels labels_predict = network.apply_regression(features_test);
//![train_and_apply]

//![get_params]
DoubleMatrix parameters = network.get_parameters();
//![get_params]

//![evaluate_error]
MeanSquaredError err = new MeanSquaredError();
double mse = err.evaluate(labels_predict, labels_test);
//![evaluate_error]

// additional integration testing variables
DoubleMatrix output = labels_predict.get_labels();

// Serialize output for integration testing (automatically generated)
WrappedObjectArray __sg_storage = new WrappedObjectArray();
SerializableAsciiFile __sg_storage_file = new SerializableAsciiFile("feedforward_net_regression.dat", 'w');
__sg_storage.append_wrapped_int(dimensions, "dimensions");
__sg_storage.append_wrapped_real_vector(parameters, "parameters");
__sg_storage.append_wrapped_real(mse, "mse");
__sg_storage.append_wrapped_real_vector(output, "output");
__sg_storage.save_serializable(__sg_storage_file);

}
}
