import org.jblas.DoubleMatrix;
import org.jblas.FloatMatrix;

import org.shogun.modshogun;
import org.shogun.MeanSquaredError;
import org.shogun.RealVector;
import org.shogun.KRRNystrom;
import org.shogun.WrappedObjectArray;
import org.shogun.SerializableAsciiFile;
import org.shogun.RegressionLabels;
import org.shogun.CSVFile;
import org.shogun.GaussianKernel;
import org.shogun.RealFeatures;

public class krr_nystrom {
static {
System.loadLibrary("modshogun");
}

public static void main(String argv[]) {
modshogun.init_shogun_with_defaults();

CSVFile f_feats_train = new CSVFile("../../data/regression_1d_sinc_features_train.dat");
CSVFile f_feats_test = new CSVFile("../../data/regression_1d_sinc_features_test.dat");
CSVFile f_labels_train = new CSVFile("../../data/regression_1d_sinc_labels_train.dat");
CSVFile f_labels_test = new CSVFile("../../data/regression_1d_sinc_labels_test.dat");

//![create_features]
RealFeatures features_train = new RealFeatures(f_feats_train);
RealFeatures features_test = new RealFeatures(f_feats_test);
RegressionLabels labels_train = new RegressionLabels(f_labels_train);
RegressionLabels labels_test = new RegressionLabels(f_labels_test);
//![create_features]

//![create_appropriate_kernel]
double width = 1.0;
GaussianKernel kernel = new GaussianKernel(features_train, features_train, width);
//![create_appropriate_kernel]

//![create_instance]
double tau = 0.001;
int m = 100;
KRRNystrom nystrom = new KRRNystrom(tau, m, kernel, labels_train);
//![create_instance]

//![train_and_apply]
nystrom.train();
RegressionLabels labels_predict = nystrom.apply_regression(features_test);
//![train_and_apply]

//![extract_alpha]
DoubleMatrix alpha = nystrom.get_alphas();
//![extract_alpha]

//![evaluate_error]
MeanSquaredError eval = new MeanSquaredError();
double mse = eval.evaluate(labels_predict, labels_test);
//![evaluate_error]

// integration testing variables
DoubleMatrix output = labels_test.get_labels();

// Serialize output for integration testing (automatically generated)
WrappedObjectArray __sg_storage = new WrappedObjectArray();
SerializableAsciiFile __sg_storage_file = new SerializableAsciiFile("krr_nystrom.dat", 'w');
__sg_storage.append_wrapped_real(width, "width");
__sg_storage.append_wrapped_real(tau, "tau");
__sg_storage.append_wrapped_int(m, "m");
__sg_storage.append_wrapped_real_vector(alpha, "alpha");
__sg_storage.append_wrapped_real(mse, "mse");
__sg_storage.append_wrapped_real_vector(output, "output");
__sg_storage.save_serializable(__sg_storage_file);

}
}
