import org.jblas.DoubleMatrix;
import org.jblas.FloatMatrix;

import org.shogun.modshogun;
import org.shogun.MeanSquaredError;
import org.shogun.PruneVarSubMean;
import org.shogun.NormOne;
import org.shogun.WrappedObjectArray;
import org.shogun.SerializableAsciiFile;
import org.shogun.RegressionLabels;
import org.shogun.CSVFile;
import org.shogun.RealVector;
import org.shogun.LeastAngleRegression;
import org.shogun.RealFeatures;

public class least_angle_regression {
static {
System.loadLibrary("modshogun");
}

public static void main(String argv[]) {
modshogun.init_shogun_with_defaults();

CSVFile f_feats_train = new CSVFile("../../data/regression_1d_linear_features_train.dat");
CSVFile f_feats_test = new CSVFile("../../data/regression_1d_linear_features_test.dat");
CSVFile f_labels_train = new CSVFile("../../data/regression_1d_linear_labels_train.dat");
CSVFile f_labels_test = new CSVFile("../../data/regression_1d_linear_labels_test.dat");

//![create_features]
RealFeatures features_train = new RealFeatures(f_feats_train);
RealFeatures features_test = new RealFeatures(f_feats_test);
RegressionLabels labels_train = new RegressionLabels(f_labels_train);
RegressionLabels labels_test = new RegressionLabels(f_labels_test);
//![create_features]

//![preprocess_features]
PruneVarSubMean SubMean = new PruneVarSubMean();
NormOne Normalize = new NormOne();
SubMean.init(features_train);
SubMean.apply_to_feature_matrix(features_train);
SubMean.apply_to_feature_matrix(features_test);
Normalize.init(features_train);
Normalize.apply_to_feature_matrix(features_train);
Normalize.apply_to_feature_matrix(features_test);
//![preprocess_features]

//![create_instance]
double lamda1 = 0.01;
LeastAngleRegression lars = new LeastAngleRegression(false);
lars.set_features(features_train);
lars.set_labels(labels_train);
lars.set_max_l1_norm(lamda1);
//![create_instance]

//![train_and_apply]
lars.train();
RegressionLabels labels_predict = lars.apply_regression(features_test);

//[!extract_w]
DoubleMatrix weights = lars.get_w();
//[!extract_w]

//![evaluate_error]
MeanSquaredError eval = new MeanSquaredError();
double mse = eval.evaluate(labels_predict, labels_test);
//![evaluate_error]

// integration testing variables
DoubleMatrix output = labels_test.get_labels();


// Serialize output for integration testing (automatically generated)
WrappedObjectArray __sg_storage = new WrappedObjectArray();
SerializableAsciiFile __sg_storage_file = new SerializableAsciiFile("least_angle_regression.dat", 'w');
__sg_storage.append_wrapped_real(lamda1, "lamda1");
__sg_storage.append_wrapped_real_vector(weights, "weights");
__sg_storage.append_wrapped_real(mse, "mse");
__sg_storage.append_wrapped_real_vector(output, "output");
__sg_storage.save_serializable(__sg_storage_file);

}
}
