import org.jblas.DoubleMatrix;
import org.jblas.FloatMatrix;

import org.shogun.modshogun;
import org.shogun.MeanSquaredError;
import org.shogun.LibSVR;
import org.shogun.RealVector;
import org.shogun.LIBSVR_SOLVER_TYPE;
import org.shogun.WrappedObjectArray;
import org.shogun.SerializableAsciiFile;
import org.shogun.RegressionLabels;
import org.shogun.CSVFile;
import org.shogun.GaussianKernel;
import org.shogun.RealFeatures;

public class support_vector_regression {
static {
System.loadLibrary("modshogun");
}

public static void main(String argv[]) {
modshogun.init_shogun_with_defaults();

CSVFile f_feats_train = new CSVFile("../../data/regression_1d_sinc_features_train.dat");
CSVFile f_feats_test = new CSVFile("../../data/regression_1d_sinc_features_test.dat");
CSVFile f_labels_train = new CSVFile("../../data/regression_1d_sinc_labels_train.dat");
CSVFile f_labels_test = new CSVFile("../../data/regression_1d_sinc_labels_test.dat");

//![create_features]
RealFeatures features_train = new RealFeatures(f_feats_train);
RealFeatures features_test = new RealFeatures(f_feats_test);
RegressionLabels labels_train = new RegressionLabels(f_labels_train);
RegressionLabels labels_test = new RegressionLabels(f_labels_test);
//![create_features]

//![create_appropriate_kernel]
double width = 1.0;
GaussianKernel kernel = new GaussianKernel(width);
//![create_appropriate_kernel]

//![create_instance]
double svm_c = 1.0;
double svr_param = 0.1;
LibSVR svr = new LibSVR(svm_c, svr_param, kernel, labels_train, LIBSVR_SOLVER_TYPE.LIBSVR_EPSILON_SVR);
//![create_instance]

//![train_and_apply]
svr.train(features_train);
RegressionLabels labels_predict = svr.apply_regression(features_test);
//![train_and_apply]

//![extract_alpha]
DoubleMatrix alpha = svr.get_alphas();
//![extract_alpha]

//![evaluate_error]
MeanSquaredError eval = new MeanSquaredError();
double mse = eval.evaluate(labels_predict, labels_test);
//![evaluate_error]

// integration testing variables
DoubleMatrix output = labels_test.get_labels();

// Serialize output for integration testing (automatically generated)
WrappedObjectArray __sg_storage = new WrappedObjectArray();
SerializableAsciiFile __sg_storage_file = new SerializableAsciiFile("support_vector_regression.dat", 'w');
__sg_storage.append_wrapped_real(width, "width");
__sg_storage.append_wrapped_real(svm_c, "svm_c");
__sg_storage.append_wrapped_real(svr_param, "svr_param");
__sg_storage.append_wrapped_real_vector(alpha, "alpha");
__sg_storage.append_wrapped_real(mse, "mse");
__sg_storage.append_wrapped_real_vector(output, "output");
__sg_storage.save_serializable(__sg_storage_file);

}
}
