import org.jblas.DoubleMatrix;
import org.jblas.FloatMatrix;

import org.shogun.modshogun;
import org.shogun.CombinedKernel;
import org.shogun.RealVector;
import org.shogun.LinearTimeMMD;
import org.shogun.GaussianBlobsDataGenerator;
import org.shogun.EKernelSelectionMethod;
import org.shogun.SerializableAsciiFile;
import org.shogun.WrappedObjectArray;
import org.shogun.GaussianKernel;

public class linear_time_mmd {
static {
System.loadLibrary("modshogun");
}

public static void main(String argv[]) {
modshogun.init_shogun_with_defaults();

GaussianBlobsDataGenerator features_p = new GaussianBlobsDataGenerator();
GaussianBlobsDataGenerator features_q = new GaussianBlobsDataGenerator();

//![create_instance]
LinearTimeMMD mmd = new LinearTimeMMD();
GaussianKernel kernel = new GaussianKernel(10, 1);
mmd.set_kernel(kernel);
mmd.set_p(features_p);
mmd.set_q(features_q);
mmd.set_num_samples_p(1000);
mmd.set_num_samples_q(1000);
double alpha = 0.05;
//![create_instance]

//![set_burst]
mmd.set_num_blocks_per_burst(1000);
//![set_burst]

//![estimate_mmd]
double statistic = mmd.compute_statistic();
//![estimate_mmd]

//![perform_test]
double threshold = mmd.compute_threshold(alpha);
double p_value = mmd.compute_p_value(statistic);
//![perform_test]

//![add_kernels]
GaussianKernel kernel1 = new GaussianKernel(10, 0.1);
GaussianKernel kernel2 = new GaussianKernel(10, 1);
GaussianKernel kernel3 = new GaussianKernel(10, 10);
mmd.add_kernel(kernel1);
mmd.add_kernel(kernel2);
mmd.add_kernel(kernel3);
//![add_kernels]

//![enable_train_test_mode]
mmd.set_train_test_mode(true);
mmd.set_train_test_ratio(1);
//![enable_train_test_mode]

//![select_kernel_single]
mmd.set_kernel_selection_strategy(EKernelSelectionMethod.KSM_MAXIMIZE_POWER);
mmd.select_kernel();
GaussianKernel learnt_kernel_single = GaussianKernel.obtain_from_generic(mmd.get_kernel());
double width = learnt_kernel_single.get_width();
//![select_kernel_single]

//![select_kernel_combined]
mmd.set_kernel_selection_strategy(EKernelSelectionMethod.KSM_MAXIMIZE_POWER, true);
mmd.select_kernel();
CombinedKernel learnt_kernel_combined = CombinedKernel.obtain_from_generic(mmd.get_kernel());
DoubleMatrix weights = learnt_kernel_combined.get_subkernel_weights();
//![select_kernel_combined]

//![perform_test_optimized]
double statistic_optimized = mmd.compute_statistic();
double p_value_optimized = mmd.compute_p_value(statistic);
//![perform_test_optimized]

// Serialize output for integration testing (automatically generated)
WrappedObjectArray __sg_storage = new WrappedObjectArray();
SerializableAsciiFile __sg_storage_file = new SerializableAsciiFile("linear_time_mmd.dat", 'w');
__sg_storage.append_wrapped_real(alpha, "alpha");
__sg_storage.append_wrapped_real(statistic, "statistic");
__sg_storage.append_wrapped_real(threshold, "threshold");
__sg_storage.append_wrapped_real(p_value, "p_value");
__sg_storage.append_wrapped_real(width, "width");
__sg_storage.append_wrapped_real_vector(weights, "weights");
__sg_storage.append_wrapped_real(statistic_optimized, "statistic_optimized");
__sg_storage.append_wrapped_real(p_value_optimized, "p_value_optimized");
__sg_storage.save_serializable(__sg_storage_file);

}
}
