import org.jblas.DoubleMatrix;
import org.jblas.FloatMatrix;

import org.shogun.modshogun;
import org.shogun.CombinedKernel;
import org.shogun.RealVector;
import org.shogun.QuadraticTimeMMD;
import org.shogun.ENullApproximationMethod;
import org.shogun.EKernelSelectionMethod;
import org.shogun.CSVFile;
import org.shogun.SerializableAsciiFile;
import org.shogun.EStatisticType;
import org.shogun.MultiKernelQuadraticTimeMMD;
import org.shogun.GaussianKernel;
import org.shogun.RealFeatures;
import org.shogun.WrappedObjectArray;

public class quadratic_time_mmd {
static {
System.loadLibrary("modshogun");
}

public static void main(String argv[]) {
modshogun.init_shogun_with_defaults();

CSVFile f_features_p = new CSVFile("../../data/two_sample_test_gaussian.dat");
CSVFile f_features_q = new CSVFile("../../data/two_sample_test_laplace.dat");

//![create_features]
RealFeatures features_p = new RealFeatures(f_features_p);
RealFeatures features_q = new RealFeatures(f_features_q);
//![create_features]

//![create_instance]
QuadraticTimeMMD mmd = new QuadraticTimeMMD(features_p, features_q);
GaussianKernel kernel = new GaussianKernel(10, 1);
mmd.set_kernel(kernel);
double alpha = 0.05;
//![create_instance]

//![estimate_mmd]
mmd.set_statistic_type(EStatisticType.ST_BIASED_FULL);
double statistic = mmd.compute_statistic();
//![estimate_mmd]

//![perform_test]
mmd.set_null_approximation_method(ENullApproximationMethod.NAM_PERMUTATION);
mmd.set_num_null_samples(200);
double threshold = mmd.compute_threshold(alpha);
double p_value = mmd.compute_p_value(statistic);
//![perform_test]

//![add_kernels]
GaussianKernel kernel1 = new GaussianKernel(10, 0.1);
GaussianKernel kernel2 = new GaussianKernel(10, 1);
GaussianKernel kernel3 = new GaussianKernel(10, 10);
mmd.add_kernel(kernel1);
mmd.add_kernel(kernel2);
mmd.add_kernel(kernel3);
//![add_kernels]

//![multi_kernel]
MultiKernelQuadraticTimeMMD mk = mmd.multikernel();
mk.add_kernel(kernel1);
mk.add_kernel(kernel2);
mk.add_kernel(kernel2);

DoubleMatrix mk_statistic = mk.compute_statistic();
DoubleMatrix mk_p_value = mk.compute_p_value();
//![multi_kernel]

//![enable_train_test_mode]
mmd.set_train_test_mode(true);
mmd.set_train_test_ratio(1);
//![enable_train_test_mode]

//![select_kernel_single]
int num_runs = 1;
int num_folds = 3;
mmd.set_kernel_selection_strategy(EKernelSelectionMethod.KSM_CROSS_VALIDATION, num_runs, num_folds, alpha);
mmd.select_kernel();
GaussianKernel learnt_kernel_single = GaussianKernel.obtain_from_generic(mmd.get_kernel());
double width = learnt_kernel_single.get_width();
//![select_kernel_single]

//![select_kernel_combined]
mmd.set_kernel_selection_strategy(EKernelSelectionMethod.KSM_MAXIMIZE_MMD, true);
mmd.select_kernel();
CombinedKernel learnt_kernel_combined = CombinedKernel.obtain_from_generic(mmd.get_kernel());
DoubleMatrix weights = learnt_kernel_combined.get_subkernel_weights();
//![select_kernel_combined]

//![perform_test_optimized]
double statistic_optimized = mmd.compute_statistic();
double p_value_optimized = mmd.compute_p_value(statistic);
//![perform_test_optimized]

// Serialize output for integration testing (automatically generated)
WrappedObjectArray __sg_storage = new WrappedObjectArray();
SerializableAsciiFile __sg_storage_file = new SerializableAsciiFile("quadratic_time_mmd.dat", 'w');
__sg_storage.append_wrapped_real(alpha, "alpha");
__sg_storage.append_wrapped_real(statistic, "statistic");
__sg_storage.append_wrapped_real(threshold, "threshold");
__sg_storage.append_wrapped_real(p_value, "p_value");
__sg_storage.append_wrapped_real_vector(mk_statistic, "mk_statistic");
__sg_storage.append_wrapped_real_vector(mk_p_value, "mk_p_value");
__sg_storage.append_wrapped_int(num_runs, "num_runs");
__sg_storage.append_wrapped_int(num_folds, "num_folds");
__sg_storage.append_wrapped_real(width, "width");
__sg_storage.append_wrapped_real_vector(weights, "weights");
__sg_storage.append_wrapped_real(statistic_optimized, "statistic_optimized");
__sg_storage.append_wrapped_real(p_value_optimized, "p_value_optimized");
__sg_storage.save_serializable(__sg_storage_file);

}
}
